'''
caching_data.py - The module contains the CachingData class.

Copyright (c) 2012 Intra2net AG
Author: Plamen Dimitrov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
'''

import pickle

class CachingData:
    """This class is responsible for the caching of data."""

    def __init__(self):
        try:
            cachingfile = open('caching_data.pkl', 'rb')
            self.data = pickle.load(cachingfile)
            #print(len(self.data), "users found.")
            self.save_flag = {}
            for user in self.data:
                self.save_flag[user] = {}
                for uid_key in self.data[user]:
                    self.save_flag[user][uid_key] = False
        except IOError:
            self.data = {}
            self.save_flag = {}
            with open('caching_data.pkl', 'wb') as cachingfile:
                pickle.dump(self.data, cachingfile)

    def __del__(self):
        with open('caching_data.pkl', 'wb') as cachingfile:
            # prepare data based on a save flag
            for user in self.save_flag:
                for uid_key in self.save_flag[user]:
                    if(not self.save_flag[user][uid_key]):
                        del self.data[user][uid_key]
                        #print(uidvalidity, "deleted from cache.")
                if(len(self.data[user])==0):
                    del self.data[user]
                    #print(user, "deleted from cache.")

            # serialize in file
            pickle.dump(self.data, cachingfile)

        #print(len(self.data), "users stored.")
    
    def _cache_new_mailbox(self, username, uid_key):
        """Store the mailbox as integer uidvalidity"""
        if(username not in self.data):
            self.data[username] = {}
            self.save_flag[username] = {}
            #print(username, "created.")
        if(uid_key not in self.data[username]):
            self.data[username][uid_key] = []
            self.save_flag[username][uid_key] = False
            #print(uid_key, "created.")
        return

    def sync_cached_mailbox(self, username, uid_key, list_ids):
        """Adds new messages to the cache and returns a list of them.
        Confirm the changes to a mailbox to finally save it."""
        new_ids = []

        if(username not in self.data or \
           uid_key not in self.data[username]):
            self._cache_new_mailbox(username, uid_key)
            new_ids = list_ids
        else:
            for uid in list_ids:
                try:
                    self.data[username][uid_key].index(uid)
                    #print("found", uid, uid_key)
                except ValueError:
                    #print("new", uid, uid_key)
                    new_ids.append(uid)

        # update cached_mailbox
        self.data[username][uid_key] = list_ids

        return new_ids

    def commit_cached_mailbox(self, username, uid_key):
        """Confirm the chages to the cached mailbox."""
        self.save_flag[username][uid_key] = True
        #print(username, uid_key, "committed.") 
        return
